@@ -649,7 +649,7 @@ public class Replace extends MatchingTask {
         logFilterChain(src.getPath());
 
         try {
-            File temp = FILE_UTILS.createTempFile("rep", ".tmp",
+            File temp = FILE_UTILS.createTempFile(getProject(), "rep", ".tmp",
                     src.getParentFile(), false, true);
             try {
                 try (FileInput in = new FileInput(src);
