@@ -30,6 +30,7 @@ import org.apache.tools.ant.taskdefs.optional.junitlauncher.confined.TestClasses
 import org.apache.tools.ant.taskdefs.optional.junitlauncher.confined.TestDefinition;
 import org.apache.tools.ant.util.FileUtils;
 import org.apache.tools.ant.util.KeepAliveOutputStream;
+import org.apache.tools.ant.util.LeadPipeInputStream;
 import org.junit.platform.engine.Filter;
 import org.junit.platform.engine.discovery.DiscoverySelectors;
 import org.junit.platform.launcher.EngineFilter;
@@ -37,6 +38,7 @@ import org.junit.platform.launcher.Launcher;
 import org.junit.platform.launcher.LauncherDiscoveryRequest;
 import org.junit.platform.launcher.TagFilter;
 import org.junit.platform.launcher.TestExecutionListener;
+import org.junit.platform.launcher.TestIdentifier;
 import org.junit.platform.launcher.TestPlan;
 import org.junit.platform.launcher.core.LauncherDiscoveryRequestBuilder;
 import org.junit.platform.launcher.core.LauncherFactory;
@@ -49,10 +51,11 @@ import java.io.OutputStream;
 import java.io.PipedInputStream;
 import java.io.PipedOutputStream;
 import java.io.PrintStream;
-import java.io.PrintWriter;
+import java.io.UncheckedIOException;
 import java.nio.file.Files;
 import java.nio.file.Path;
 import java.nio.file.Paths;
+import java.text.NumberFormat;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
@@ -128,7 +131,7 @@ public class LauncherSupport {
                     final List<TestExecutionListener> testExecutionListeners = new ArrayList<>();
                     // a listener that we always put at the front of list of listeners
                     // for this request.
-                    final Listener firstListener = new Listener();
+                    final Listener firstListener = new Listener(System.out);
                     // we always enroll the summary generating listener, to the request, so that we
                     // get to use some of the details of the summary for our further decision making
                     testExecutionListeners.add(firstListener);
@@ -136,9 +139,9 @@ public class LauncherSupport {
                     final PrintStream originalSysOut = System.out;
                     final PrintStream originalSysErr = System.err;
                     try {
-                        firstListener.switchedSysOutHandle = trySwitchSysOutErr(testRequest, StreamType.SYS_OUT);
-                        firstListener.switchedSysErrHandle = trySwitchSysOutErr(testRequest, StreamType.SYS_ERR);
-                        launcher.execute(request, testExecutionListeners.toArray(new TestExecutionListener[testExecutionListeners.size()]));
+                        firstListener.switchedSysOutHandle = trySwitchSysOutErr(testRequest, StreamType.SYS_OUT, originalSysErr);
+                        firstListener.switchedSysErrHandle = trySwitchSysOutErr(testRequest, StreamType.SYS_ERR, originalSysErr);
+                        launcher.execute(request, testExecutionListeners.toArray(new TestExecutionListener[0]));
                     } finally {
                         // switch back sysout/syserr to the original
                         try {
@@ -151,6 +154,29 @@ public class LauncherSupport {
                         } catch (Exception e) {
                             // ignore
                         }
+                        // close the streams that we had used to redirect System.out/System.err
+                        try {
+                            firstListener.switchedSysOutHandle.ifPresent((h) -> {
+                                try {
+                                    h.close();
+                                } catch (Exception e) {
+                                    // ignore
+                                }
+                            });
+                        } catch (Exception e) {
+                            // ignore
+                        }
+                        try {
+                            firstListener.switchedSysErrHandle.ifPresent((h) -> {
+                                try {
+                                    h.close();
+                                } catch (Exception e) {
+                                    // ignore
+                                }
+                            });
+                        } catch (Exception e) {
+                            // ignore
+                        }
                     }
                     handleTestExecutionCompletion(test, firstListener.getSummary());
                 } finally {
@@ -226,6 +252,7 @@ public class LauncherSupport {
         testRequest.closeUponCompletion(resultFormatter);
         // set the execution context
         resultFormatter.setContext(this.testExecutionContext);
+        resultFormatter.setUseLegacyReportingName(formatterDefinition.isUseLegacyReportingName());
         // set the destination output stream for writing out the formatted result
         final java.nio.file.Path resultOutputFile = getListenerOutputFile(testRequest, formatterDefinition);
         try {
@@ -255,13 +282,7 @@ public class LauncherSupport {
             final StringBuilder sb = new StringBuilder("TEST-");
             sb.append(testRequest.getName() == null ? "unknown" : testRequest.getName());
             sb.append(".");
-            final String suffix;
-            if ("org.apache.tools.ant.taskdefs.optional.junitlauncher.LegacyXmlResultFormatter".equals(listener.getClassName())) {
-                suffix = "xml";
-            } else {
-                suffix = "txt";
-            }
-            sb.append(suffix);
+            sb.append(listener.getExtension());
             filename = sb.toString();
         }
         if (listener.getOutputDir() != null) {
@@ -293,17 +314,13 @@ public class LauncherSupport {
             throw new BuildException("Listener class " + className + " is not of type " + TestExecutionListener.class.getName());
         }
         try {
-            return (TestExecutionListener) klass.newInstance();
+            return (TestExecutionListener) klass.getDeclaredConstructor().newInstance();
         } catch (Exception e) {
             throw new BuildException("Failed to create an instance of listener " + className, e);
         }
     }
 
     private void handleTestExecutionCompletion(final TestDefinition test, final TestExecutionSummary summary) {
-        if (this.launchDefinition.isPrintSummary()) {
-            // print the summary to System.out
-            summary.printTo(new PrintWriter(System.out, true));
-        }
         final boolean hasTestFailures = summary.getTotalFailureCount() != 0;
         if (hasTestFailures) {
             // keep track of the test failure(s) for the entire launched instance
@@ -333,7 +350,8 @@ public class LauncherSupport {
         }
     }
 
-    private Optional<SwitchedStreamHandle> trySwitchSysOutErr(final TestRequest testRequest, final StreamType streamType) {
+    private Optional<SwitchedStreamHandle> trySwitchSysOutErr(final TestRequest testRequest, final StreamType streamType,
+                                                              final PrintStream originalSysErr) {
         switch (streamType) {
             case SYS_OUT: {
                 if (!testRequest.interestedInSysOut()) {
@@ -356,7 +374,7 @@ public class LauncherSupport {
         final PipedOutputStream pipedOutputStream = new PipedOutputStream();
         final PipedInputStream pipedInputStream;
         try {
-            pipedInputStream = new PipedInputStream(pipedOutputStream);
+            pipedInputStream = new LeadPipeInputStream(pipedOutputStream);
         } catch (IOException ioe) {
             // log and return
             return Optional.empty();
@@ -367,22 +385,32 @@ public class LauncherSupport {
             case SYS_OUT: {
                 System.setOut(new PrintStream(printStream));
                 streamer = new SysOutErrStreamReader(this, pipedInputStream,
-                        StreamType.SYS_OUT, testRequest.getSysOutInterests());
+                        StreamType.SYS_OUT, testRequest.getSysOutInterests(), originalSysErr);
                 final Thread sysOutStreamer = new Thread(streamer);
                 sysOutStreamer.setDaemon(true);
                 sysOutStreamer.setName("junitlauncher-sysout-stream-reader");
-                sysOutStreamer.setUncaughtExceptionHandler((t, e) -> this.log("Failed in sysout streaming", e, Project.MSG_INFO));
+                sysOutStreamer.setUncaughtExceptionHandler((t, e) -> {
+                    // skip the logging redirection infrastructure of junitlauncher task (which is what has
+                    // failed here) and instead directly write out the error to the original System.err
+                    originalSysErr.println("Failed in sysout streaming");
+                    e.printStackTrace(originalSysErr);
+                });
                 sysOutStreamer.start();
                 break;
             }
             case SYS_ERR: {
                 System.setErr(new PrintStream(printStream));
                 streamer = new SysOutErrStreamReader(this, pipedInputStream,
-                        StreamType.SYS_ERR, testRequest.getSysErrInterests());
+                        StreamType.SYS_ERR, testRequest.getSysErrInterests(), originalSysErr);
                 final Thread sysErrStreamer = new Thread(streamer);
                 sysErrStreamer.setDaemon(true);
                 sysErrStreamer.setName("junitlauncher-syserr-stream-reader");
-                sysErrStreamer.setUncaughtExceptionHandler((t, e) -> this.log("Failed in syserr streaming", e, Project.MSG_INFO));
+                sysErrStreamer.setUncaughtExceptionHandler((t, e) -> {
+                    // skip the logging redirection infrastructure of junitlauncher task (which is what has
+                    // failed here) and instead directly write out the error to the original System.err
+                    originalSysErr.println("Failed in syserr streaming");
+                    e.printStackTrace(originalSysErr);
+                });
                 sysErrStreamer.start();
                 break;
             }
@@ -479,20 +507,26 @@ public class LauncherSupport {
         SYS_ERR
     }
 
+    // Implementation note: Logging from this class is prohibited since it can lead
+    // to deadlocks (see bz-64733 for details)
     private static final class SysOutErrStreamReader implements Runnable {
         private static final byte[] EMPTY = new byte[0];
 
         private final LauncherSupport launchManager;
+        private final PrintStream originalSysErr;
         private final InputStream sourceStream;
         private final StreamType streamType;
         private final Collection<TestResultFormatter> resultFormatters;
         private volatile SysOutErrContentDeliverer contentDeliverer;
 
-        SysOutErrStreamReader(final LauncherSupport launchManager, final InputStream source, final StreamType streamType, final Collection<TestResultFormatter> resultFormatters) {
+        SysOutErrStreamReader(final LauncherSupport launchManager, final InputStream source,
+                              final StreamType streamType, final Collection<TestResultFormatter> resultFormatters,
+                              final PrintStream originalSysErr) {
             this.launchManager = launchManager;
             this.sourceStream = source;
             this.streamType = streamType;
             this.resultFormatters = resultFormatters;
+            this.originalSysErr = originalSysErr;
         }
 
         @Override
@@ -511,8 +545,8 @@ public class LauncherSupport {
                     streamContentDeliver.availableData.offer(copy);
                 }
             } catch (IOException e) {
-                this.launchManager.log("Failed while streaming " + (this.streamType == StreamType.SYS_OUT ? "sysout" : "syserr") + " data",
-                        e, Project.MSG_INFO);
+                // let the UncaughtExceptionHandler of this thread deal with this exception
+                throw new UncheckedIOException(e);
             } finally {
                 streamContentDeliver.stop = true;
                 // just "wakeup" the delivery thread, to take into account
@@ -584,7 +618,7 @@ public class LauncherSupport {
         }
     }
 
-    private final class SwitchedStreamHandle {
+    private final class SwitchedStreamHandle implements AutoCloseable {
         private final PipedOutputStream outputStream;
         private final SysOutErrStreamReader streamReader;
 
@@ -592,15 +626,64 @@ public class LauncherSupport {
             this.streamReader = streamReader;
             this.outputStream = outputStream;
         }
+
+        @Override
+        public void close() throws Exception {
+            outputStream.close();
+            streamReader.sourceStream.close();
+        }
     }
 
     private final class Listener extends SummaryGeneratingListener {
+        private final PrintStream originalSysOut;
+
         private Optional<SwitchedStreamHandle> switchedSysOutHandle;
         private Optional<SwitchedStreamHandle> switchedSysErrHandle;
 
+        private Listener(final PrintStream originalSysOut) {
+            this.originalSysOut = originalSysOut;
+        }
+
+        @Override
+        public void executionStarted(final TestIdentifier testIdentifier) {
+            super.executionStarted(testIdentifier);
+            AbstractJUnitResultFormatter.isTestClass(testIdentifier).ifPresent(testClass ->
+                    this.originalSysOut.println("Running " + testClass.getClassName()));
+        }
+
+
+        private static final double ONE_SECOND = 1000.0;
+        // We use this only in the testPlanExecutionFinished method, which
+        // as per the JUnit5 platform semantics won't be called concurrently
+        // by multiple threads (https://github.com/junit-team/junit5/issues/2539#issuecomment-766325555).
+        // So it's safe to use this without any additional thread safety access controls.
+        private NumberFormat timeFormatter = NumberFormat.getInstance();
+
         @Override
         public void testPlanExecutionFinished(final TestPlan testPlan) {
             super.testPlanExecutionFinished(testPlan);
+            if (!testPlan.containsTests()) {
+                // we print the summary only if any tests are present
+                return;
+            }
+            if (launchDefinition.isPrintSummary()) {
+                final TestExecutionSummary summary = this.getSummary();
+                // Keep the summary as close to as the old junit task summary
+                // tests run, failed, skipped, duration
+                final StringBuilder sb = new StringBuilder("Tests run: ");
+                sb.append(summary.getTestsStartedCount());
+                sb.append(", Failures: ");
+                sb.append(summary.getTestsFailedCount());
+                sb.append(", Aborted: ");
+                sb.append(summary.getTestsAbortedCount());
+                sb.append(", Skipped: ");
+                sb.append(summary.getTestsSkippedCount());
+                sb.append(", Time elapsed: ");
+                final long elapsedMs = summary.getTimeFinished() - summary.getTimeStarted();
+                sb.append(timeFormatter.format(elapsedMs / ONE_SECOND));
+                sb.append(" sec");
+                this.originalSysOut.println(sb.toString());
+            }
             // now that the test plan execution is finished, close the switched sysout/syserr output streams
             // and wait for the sysout and syserr content delivery, to result formatters, to finish
             if (this.switchedSysOutHandle.isPresent()) {
