@@ -18,9 +18,17 @@
 
 package org.apache.tools.ant.taskdefs.optional.ssh;
 
+import java.io.File;
+import java.io.IOException;
+import java.util.HashMap;
+import java.util.Map;
+
+import com.jcraft.jsch.ConfigRepository;
+import com.jcraft.jsch.OpenSSHConfig;
 import org.apache.tools.ant.BuildException;
 import org.apache.tools.ant.Project;
 import org.apache.tools.ant.Task;
+import org.apache.tools.ant.types.Environment.Variable;
 
 import com.jcraft.jsch.JSch;
 import com.jcraft.jsch.JSchException;
@@ -42,8 +50,10 @@ public abstract class SSHBase extends Task implements LogListener {
     private boolean failOnError = true;
     private boolean verbose;
     private final SSHUserInfo userInfo;
+    private String sshConfig;
     private int serverAliveCountMax = 3;
     private int serverAliveIntervalSeconds = 0;
+    private final Map<String, String> additionalConfig = new HashMap<>();
 
     /**
      * Constructor for SSHBase.
@@ -106,6 +116,24 @@ public abstract class SSHBase extends Task implements LogListener {
         return verbose;
     }
 
+    /**
+     * Get the OpenSSH config file (~/.ssh/config).
+     * @return the OpenSSH config file
+     * @since Ant 1.10.8
+     */
+    public String getSshConfig() {
+        return sshConfig;
+    }
+
+    /**
+     * Set the OpenSSH config file (~/.ssh/config).
+     * @param sshConfig the OpenSSH config file
+     * @since Ant 1.10.8
+     */
+    public void setSshConfig(String sshConfig) {
+        this.sshConfig = sshConfig;
+    }
+
     /**
      * Set the serverAliveCountMax value.
      * @param countMax int
@@ -223,6 +251,10 @@ public abstract class SSHBase extends Task implements LogListener {
         return port;
     }
 
+    public void addConfiguredAdditionalConfig(final Variable v) {
+        additionalConfig.put(v.getKey(), v.getValue());
+    }
+
     /**
      * Initialize the task.
      * This initializes the known hosts and sets the default port.
@@ -235,6 +267,39 @@ public abstract class SSHBase extends Task implements LogListener {
         this.port = SSH_PORT;
     }
 
+    /**
+     * Load the SSH configuration file.
+     * @throws BuildException on error
+     */
+    protected void loadSshConfig() throws BuildException {
+        if (sshConfig != null && (userInfo.getName() == null || userInfo.getKeyfile() == null)) {
+            if (!new File(sshConfig).exists()) {
+                throw new BuildException("The SSH configuration file specified doesn't exist: " + sshConfig);
+            }
+
+            log("Loading SSH configuration file " + sshConfig, Project.MSG_DEBUG);
+            ConfigRepository.Config config = null;
+            try {
+                config = OpenSSHConfig.parseFile(sshConfig).getConfig(host);
+            } catch (IOException e) {
+                throw new BuildException("Failed to load the SSH configuration file " + sshConfig, e);
+            }
+
+            if (config.getHostname() != null) {
+                host = config.getHostname();
+            }
+
+            if (userInfo.getName() == null) {
+                userInfo.setName(config.getUser());
+            }
+
+            if (userInfo.getKeyfile() == null) {
+                log("Using SSH key file " + config.getValue("IdentityFile") + " for host " + host, Project.MSG_INFO);
+                userInfo.setKeyfile(config.getValue("IdentityFile"));
+            }
+        }
+    }
+
     /**
      * Open an ssh session.
      * @return the opened session
@@ -275,6 +340,11 @@ public abstract class SSHBase extends Task implements LogListener {
             session.setServerAliveInterval(getServerAliveIntervalSeconds() * 1000);
         }
 
+        additionalConfig.forEach((k,v) -> {
+            log("Setting additional config value " + k, Project.MSG_DEBUG);
+            session.setConfig(k, v);
+        });
+
         log("Connecting to " + host + ":" + port);
         session.connect();
         return session;
