@@ -6,7 +6,7 @@
  *  (the "License"); you may not use this file except in compliance with
  *  the License.  You may obtain a copy of the License at
  *
- *      http://www.apache.org/licenses/LICENSE-2.0
+ *      https://www.apache.org/licenses/LICENSE-2.0
  *
  *  Unless required by applicable law or agreed to in writing, software
  *  distributed under the License is distributed on an "AS IS" BASIS,
@@ -41,7 +41,7 @@ public class ProxyDiagnostics {
     private URI destURI;
 
     /** {@value} */
-    public static final String DEFAULT_DESTINATION = "http://ant.apache.org/";
+    public static final String DEFAULT_DESTINATION = "https://ant.apache.org/";
 
     /**
      * create a diagnostics binding for a specific URI
