@@ -325,6 +325,11 @@ public class ZipOutputStream extends FilterOutputStream {
 
     private final Calendar calendarInstance = Calendar.getInstance();
 
+    /**
+     * Temporary buffer used for the {@link #write(int)} method.
+     */
+    private final byte[] oneByte = new byte[1];
+
     /**
      * Creates a new ZIP OutputStream filtering the underlying stream.
      * @param out the outputstream to zip
@@ -490,7 +495,8 @@ public class ZipOutputStream extends FilterOutputStream {
     }
 
     /**
-     * {@inheritDoc}
+     * Finish writing the archive.
+     *
      * @throws Zip64RequiredException if the archive's size exceeds 4
      * GByte or there are more than 65535 entries inside the archive
      * and {@link #setUseZip64} is {@link Zip64Mode#Never}.
@@ -723,7 +729,8 @@ public class ZipOutputStream extends FilterOutputStream {
     }
 
     /**
-     * {@inheritDoc}
+     * Put the specified entry into the archive.
+     *
      * @throws Zip64RequiredException if the entry's uncompressed or
      * compressed size is known to exceed 4 GByte and {@link #setUseZip64}
      * is {@link Zip64Mode#Never}.
@@ -901,6 +908,19 @@ public class ZipOutputStream extends FilterOutputStream {
         return ZipUtil.canHandleEntryData(ae);
     }
 
+    /**
+     * Writes a byte to ZIP entry.
+     *
+     * @param b the byte to write
+     * @throws IOException on error
+     * @since Ant 1.10.10
+     */
+    @Override
+    public void write(int b) throws IOException {
+        oneByte[0] = (byte) (b & 0xff);
+        write(oneByte, 0, 1);
+    }
+
     /**
      * Writes bytes to ZIP entry.
      *
