@@ -6,7 +6,7 @@
  *  (the "License"); you may not use this file except in compliance with
  *  the License.  You may obtain a copy of the License at
  *
- *      http://www.apache.org/licenses/LICENSE-2.0
+ *      https://www.apache.org/licenses/LICENSE-2.0
  *
  *  Unless required by applicable law or agreed to in writing, software
  *  distributed under the License is distributed on an "AS IS" BASIS,
@@ -70,7 +70,7 @@ public class BuildFileRule extends ExternalResource {
 
     /**
      * Tidies up following a test execution. If the currently configured
-     * project has a <tt>tearDown</tt> target then this will automatically
+     * project has a <code>tearDown</code> target then this will automatically
      * be called, otherwise this method will not perform any actions.
      */
     @Override
@@ -153,11 +153,14 @@ public class BuildFileRule extends ExternalResource {
         logBuffer = new StringBuffer();
         fullLogBuffer = new StringBuffer();
         project = new Project();
+        if (Boolean.getBoolean(MagicTestNames.TEST_BASEDIR_IGNORE)) {
+            System.clearProperty(MagicNames.PROJECT_BASEDIR);
+        }
         project.init();
-        File antFile = new File(System.getProperty("root"), filename);
-        project.setProperty("ant.processid", ProcessUtil.getProcessId("<Process>"));
-        project.setProperty("ant.threadname", Thread.currentThread().getName());
-        project.setUserProperty("ant.file", antFile.getAbsolutePath());
+        File antFile = new File(System.getProperty(MagicTestNames.TEST_ROOT_DIRECTORY), filename);
+        project.setProperty(MagicTestNames.TEST_PROCESS_ID, ProcessUtil.getProcessId("<Process>"));
+        project.setProperty(MagicTestNames.TEST_THREAD_NAME, Thread.currentThread().getName());
+        project.setUserProperty(MagicNames.ANT_FILE, antFile.getAbsolutePath());
         project.addBuildListener(new AntTestListener(logLevel));
         ProjectHelper.configureProject(project, antFile);
     }
